-------------------------------------------------------------------------------
-- counters.vhd - entity/architecture pair
-------------------------------------------------------------------------------
--
--  ***************************************************************************
--  **  Copyright(C) 2003 by Xilinx, Inc. All rights reserved.               **
--  **                                                                       **
--  **  This text contains proprietary, confidential                         **
--  **  information of Xilinx, Inc. , is distributed by                      **
--  **  under license from Xilinx, Inc., and may be used,                    **
--  **  copied and/or disclosed only pursuant to the terms                   **
--  **  of a valid license agreement with Xilinx, Inc.                       **
--  **                                                                       **
--  **  Unmodified source code is guaranteed to place and route,             **
--  **  function and run at speed according to the datasheet                 **
--  **  specification. Source code is provided "as-is", with no              **
--  **  obligation on the part of Xilinx to provide support.                 **
--  **                                                                       **
--  **  Xilinx Hotline support of source code IP shall only include          **
--  **  standard level Xilinx Hotline support, and will only address         **
--  **  issues and questions related to the standard released Netlist        **
--  **  version of the core (and thus indirectly, the original core source). **
--  **                                                                       **
--  **  The Xilinx Support Hotline does not have access to source            **
--  **  code and therefore cannot answer specific questions related          **
--  **  to source HDL. The Xilinx Support Hotline will only be able          **
--  **  to confirm the problem in the Netlist version of the core.           **
--  **                                                                       **
--  **  This copyright and support notice must be retained as part           **
--  **  of this text at all times.                                           **
--  ***************************************************************************
--
-------------------------------------------------------------------------------
-- Filename:        counters.vhd
-- Version:         v1.00c
-- Description:     This file contains all of the counters for the OPB_LCD
--                  
-- VHDL-Standard:   VHDL'93
-------------------------------------------------------------------------------
-- Structure:   
--                  opb_lcd.vhd
--                      -- counters.vhd
--                         -- init_statemachine.vhd
--                         -- clock_gen.vhd
--                         -- command_statemachine.vhd
--                         -- data_statemachine.vhd
--                         -- io_registers.vhd
--                         -- write_async_fifo.vhd
--                         -- fifo_read_control.vhd
--                         -- reg_unreg.vhd
--                         -- read_data_path.vhd
--                         -- multiple_datawidth.vhd
--                         -- ipic_if.vhd
--                             -- addr_cmp.vhd
--
-------------------------------------------------------------------------------
-- Author:          WT 
-- History:
--  WT              03/10/06    First Version
-------------------------------------------------------------------------------
-- Naming Conventions:
--      active low signals:                     "*_n"
--      clock signals:                          "clk", "clk_div#", "clk_#x" 
--      reset signals:                          "rst", "rst_n" 
--      generics:                               "C_*" 
--      user defined types:                     "*_TYPE" 
--      state machine next state:               "*_ns" 
--      state machine current state:            "*_cs" 
--      combinatorial signals:                  "*_com" 
--      pipelined or register delay signals:    "*_d#" 
--      counter signals:                        "*cnt*"
--      clock enable signals:                   "*_ce" 
--      internal version of output port         "*_i"
--      device pins:                            "*_pin" 
--      ports:                                  - Names begin with Uppercase 
--      processes:                              "*_PROCESS" 
--      component instantiations:               "<ENTITY_>I_<#|FUNC>
-------------------------------------------------------------------------------

library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use IEEE.std_logic_unsigned.all;

library proc_common_v2_00_a;
use proc_common_v2_00_a.proc_common_pkg.all;
use proc_common_v2_00_a.all;

-------------------------------------------------------------------------------
-- Definition of Generics:
--
-- Definition of Ports:
--  -- inputs
--      cnt_value             -- value to count to
--      cnt_load              -- strobe to load value
--
--  -- outputs
--      cnt_end               -- indicator when count was reached
--
--  -- Clocks and reset
--      Clk                 
--      Rst               
---------------------------------------------------------------------------

-----------------------------------------------------------------------------
-- Entity section
-----------------------------------------------------------------------------

entity mycounter is
  generic ( 
        C_CLK_PERIOD     : integer := 10000;
        C_CNT_SIZE       : integer := 16
        );
  port (
       cnt_value         : in  std_logic_vector(0 to C_CNT_SIZE-1);
       cnt_load          : in  std_logic;
       cnt_end           : out std_logic;            
    
       Clk               : in  std_logic;
       Rst               : in  std_logic
       );
end entity mycounter;

-----------------------------------------------------------------------------
-- Architecture section
-----------------------------------------------------------------------------

architecture imp of mycounter is

-----------------------------------------------------------------------------
-- Constant declarations
-----------------------------------------------------------------------------

constant CNT_RST     : std_logic_vector(0 to C_CNT_SIZE-1) := (others => '0');

-----------------------------------------------------------------------------
-- Signal declarations
-----------------------------------------------------------------------------

signal cur_count     : std_logic_vector(0 to C_CNT_SIZE-1);
signal cnt_enable    : std_logic;
signal cnt_end_i     : std_logic;


-----------------------------------------------------------------------------
-- Component declarations
-----------------------------------------------------------------------------

-----------------------------------------------------------------------------
-- Begin architecture
-----------------------------------------------------------------------------

begin  

-------------------------------------------------------------------------------
-- Instantiate the counters
-------------------------------------------------------------------------------

-------------------------------------------------------------------------------
-- Delay counter
  DLYCNT_I: entity proc_common_v2_00_a.counter
    generic map (
                  C_NUM_BITS     => C_CNT_SIZE
                )
    port map    ( Clk            => Clk,
                  Rst            => cnt_load,
                  Load_In        => CNT_RST, 
                  Count_Enable   => cnt_enable,
                  Count_Load     => '0',
                  Count_Down     => '0',  
                  Count_Out      => cur_count,   
                  Carry_Out      => open
                 );
                 
    DLYCNT_END_PROCESS: process (Clk)
    begin
      if Clk'event and Clk='1' then
        if Rst = RESET_ACTIVE then
          cnt_end_i  <= '0';
          cnt_enable <= '0';
        elsif cnt_load = '1' then
          cnt_end_i  <= '0';
          cnt_enable <= '1';
        elsif cnt_enable = '1' then
          if cur_count = cnt_value then
            cnt_end_i  <= '1';
          end if;
        end if;

        -- make end 1 clock wide, then disable the counter
        if cnt_end_i = '1' then
          cnt_end_i  <= '0';
          cnt_enable <= '0';
        end if;
      end if;
    end process DLYCNT_END_PROCESS;  

  cnt_end <= cnt_end_i;
         
end imp;
--------------------------------------------------------------------------------
--End of File counter.vhd
--------------------------------------------------------------------------------