--==================================================================================
-- Design unit  :  Crono Example
--
-- File name    :  crono_example.vhd
--                
-- Dependencies :  IEEE.Std_Logic_1164
--                 
--                
-- Author       :  MSeveri 
--                
-- Description  :  
--
-- Limitations  :  None
--
------------------------------------------------------------------------------------
-- Revision List
--
-- Version        Author       Date                  Changes
-- 1.0            MSev         06 February, 2008     First version
--==================================================================================


--==================================================================================
-- Library Definition
library ieee;
use ieee.std_logic_1164.all;
--==================================================================================


--==================================================================================
-- Entity Definition
--==================================================================================
entity crono_example is
port
(
-- clock and reset ------------------------------------------------------------------
  arst     : IN std_logic;
  clk      : IN std_logic;
-- control input --------------------------------------------------------------------
  button_A : IN std_logic;
  button_B : IN std_logic;
-- LCD interface --------------------------------------------------------------------
  lcd_e    : OUT std_logic;
  lcd_rw   : OUT std_logic;    
  lcd_rs   : OUT std_logic;    
  lcd_db   : OUT std_logic_vector(7 downto 0)  -- use 4-bit interface
);
end entity crono_example;
--==================================================================================

--==================================================================================
architecture struct of crono_example is
--==================================================================================
------------------------------------------------------------------------------------
-- internal signal
------------------------------------------------------------------------------------
  signal arst_i           : std_logic;
  signal clk_ce           : std_logic;
  signal time_incr        : std_logic;
  signal time_rst         : std_logic;
  signal time_hour        : std_logic_vector(7 downto 0);
  signal time_minute      : std_logic_vector(7 downto 0);
  signal time_second      : std_logic_vector(7 downto 0);
  signal time_update      : std_logic;
  
  signal button_A_on      : std_logic;
  signal button_B_on      : std_logic;
------------------------------------------------------------------------------------

begin
------------------------------------------------------------------------------------
-- Internal Reset
------------------------------------------------------------------------------------
 InternalReset : process (arst, clk)
   variable v : std_logic_vector(3 downto 0);
 begin
   if arst='1' then
     v:=x"f";
   elsif clk'event and clk='1' then
     v(3):=v(2); v(2):=v(1); v(1):=v(0); v(0):='0';
   end if;
   arst_i <= v(3);
 end process InternalReset;
------------------------------------------------------------------------------------

------------------------------------------------------------------------------------
-- Controller
------------------------------------------------------------------------------------
  ButtonA_Debouncer : entity work.debouncer 
  port map
  (
   arst	=> arst_i, clk => clk,
   d => button_A, event_on	=> button_A_On, event_off => open
  );
  
  ButtonB_Debouncer : entity work.debouncer 
  port map
  (
   arst	=> arst_i, clk => clk,
   d => button_B, event_on	=> button_B_On, event_off => open
  );
 
  CronoCtrl : entity work.crono_ctrl
    port map
    ( arst => arst_i, clk => clk,
      A => button_A_On, B => button_B_On,
      time_incr => time_incr, time_rst => time_rst 
    );   
------------------------------------------------------------------------------------

------------------------------------------------------------------------------------
-- Time Prescaler
------------------------------------------------------------------------------------
  TimePrescaler : process (arst_i, clk)
    variable cnt_v : integer;
  begin
    if arst_i='1' then
      cnt_v  :=0;
      clk_ce <= '0';
    elsif clk'event and clk='1' then
      if cnt_v=49999999 then             -- count 1 second                 
        clk_ce<='1'; cnt_v:=0;
      else
        clk_ce<='0'; cnt_v:=cnt_v+1;
      end if;
    end if;
  end process TimePrescaler;
------------------------------------------------------------------------------------

------------------------------------------------------------------------------------
-- Time Counter
------------------------------------------------------------------------------------
  TimeCounter : entity work.time_counter
    port map
    ( arst => arst_i, clk => clk,
      ce => clk_ce, en => time_incr, reset => time_rst,
      hour => time_hour, minute => time_minute, second => time_second
     );
------------------------------------------------------------------------------------

------------------------------------------------------------------------------------
-- LCD Interface
------------------------------------------------------------------------------------
 TimeUpdate : process (arst, clk)
 begin
  if arst='1' then
    Time_Update <= '0';
  elsif clk'event and clk='1' then
    if (time_incr='1' and clk_ce='1') then
      Time_Update <= '1';
    elsif time_rst='1' then
      Time_Update <= '1';     
    else
      Time_Update <= '0';         
    end if;
  end if;
 end process TimeUpdate;
 
 LCDInterface : entity work.LCD_Interface
   port map
   ( arst => arst_i, clk => clk,
     hour => time_hour, minute => time_minute, second => time_second,
     update => time_update,
     lcd_e => lcd_e, lcd_rw => lcd_rw, lcd_rs => lcd_rs, lcd_db => lcd_db(7 downto 4)
   );
	
	lcd_db(3 downto 0) <= x"f";
------------------------------------------------------------------------------------
end architecture struct;
--==================================================================================
