--==================================================================================
-- Design unit  :  LCD Manager
--
-- File name    :  lcd_manager.vhd
--                
-- Dependencies :  IEEE.Std_Logic_1164
--                 
--                
-- Author       :  MSeveri 
--                
-- Description  :  
--
-- Limitations  :  None
--
------------------------------------------------------------------------------------
-- Revision List
--
-- Version        Author       Date                  Changes
-- 1.0            MSev         06 February, 2008     First version
--==================================================================================


--==================================================================================
-- Library Definition
library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
--==================================================================================


--==================================================================================
-- Entity Definition
--==================================================================================
entity LCD_Manager is
 port 
 (
 -- clock and reset ----------------------------------------------------------------
   arst         : IN std_logic;
   clk          : IN std_logic;
 -- ctrl input ---------------------------------------------------------------------
   hour         : IN std_logic_vector(7 downto 0); 
   minute       : IN std_logic_vector(7 downto 0);
   second       : IN std_logic_vector(7 downto 0);
   update       : IN std_logic;
 -- control/data to lcd interface --------------------------------------------------
   lcd_if_reset : OUT std_logic;
   lcd_cmd_data : OUT std_logic_vector(7 downto 0);
   lcd_wrdata   : OUT std_logic;
   lcd_wrcmd    : OUT std_logic
 );  
end entity LCD_Manager;
--==================================================================================


--==================================================================================
-- Architecture description
--==================================================================================
architecture rtl of LCD_Manager is
------------------------------------------------------------------------------------
-- internal signal 
------------------------------------------------------------------------------------
  type state is (idle,
                 write_cmd, wait_write_cmd, 
                 write_data, wait_write_data
                );  

  signal state_c        : state;
  signal lcd_cmd_data_c : std_logic_vector(7 downto 0);
  signal lcd_wrdata_c   : std_logic;
  signal lcd_wrcmd_c    : std_logic;
  signal char_count_c   : std_logic_vector(3 downto 0);
  signal wait_count_c   : std_logic_vector(15 downto 0);

  signal state_r        : state;
  signal lcd_cmd_data_r : std_logic_vector(7 downto 0);
  signal lcd_wrdata_r   : std_logic;
  signal lcd_wrcmd_r    : std_logic;
  signal char_count_r   : std_logic_vector(3 downto 0);
  signal wait_count_r   : std_logic_vector(15 downto 0);
------------------------------------------------------------------------------------
begin
------------------------------------------------------------------------------------
-- State Machine - Combinatorial process
------------------------------------------------------------------------------------
 SMCombinatorial : process (update, state_r, char_count_r, wait_count_r,
                            lcd_cmd_data_r, hour, minute, second)
 begin
   case state_r is
     when idle =>
       if update='1' then 
         state_c <= write_cmd; 
       else 
         state_c <= idle;
       end if;
       lcd_cmd_data_c <= x"00";
       lcd_wrdata_c   <= '0';
       lcd_wrcmd_c    <= '0';
       char_count_c   <= x"0";
       wait_count_c   <= x"0000";
     
     when write_cmd =>
       state_c      <= wait_write_cmd;
       if char_count_r=x"0" then lcd_cmd_data_c <= (x"44"+x"80");
       elsif char_count_r=x"1" then lcd_cmd_data_c <= (x"45"+x"80");
       elsif char_count_r=x"2" then lcd_cmd_data_c <= (x"46"+x"80");
       elsif char_count_r=x"3" then lcd_cmd_data_c <= (x"47"+x"80");
       elsif char_count_r=x"4" then lcd_cmd_data_c <= (x"48"+x"80");
       elsif char_count_r=x"5" then lcd_cmd_data_c <= (x"49"+x"80");
       elsif char_count_r=x"6" then lcd_cmd_data_c <= (x"4A"+x"80");
       elsif char_count_r=x"7" then lcd_cmd_data_c <= (x"4B"+x"80");
       else lcd_cmd_data_c <= x"ff";
       end if;       
       lcd_wrdata_c   <= '0';
       lcd_wrcmd_c    <= '1';
       char_count_c   <= char_count_r;
       wait_count_c   <= x"0000";
     
     when wait_write_cmd =>
      if wait_count_r=x"c34f" then  -- 1 ms time-out
        state_c  <= write_data; 
      else
        state_c  <= wait_write_cmd;
      end if;
      lcd_cmd_data_c <= lcd_cmd_data_r;
      lcd_wrdata_c   <= '0';
      lcd_wrcmd_c    <= '0';
      char_count_c   <= char_count_r;
      wait_count_c   <= wait_count_r+1;

     when write_data =>
       state_c      <= wait_write_data;
       if char_count_r=x"0" then lcd_cmd_data_c <= x"3" & hour(7 downto 4) ;
       elsif char_count_r=x"1" then lcd_cmd_data_c <= x"3" & hour(3 downto 0); 
       elsif char_count_r=x"2" then lcd_cmd_data_c <= x"3a"; 
       elsif char_count_r=x"3" then lcd_cmd_data_c <= x"3" & minute(7 downto 4); 
       elsif char_count_r=x"4" then lcd_cmd_data_c <= x"3" & minute(3 downto 0) ; 
       elsif char_count_r=x"5" then lcd_cmd_data_c <= x"3a";
       elsif char_count_r=x"6" then lcd_cmd_data_c <= x"3" & second(7 downto 4) ; 
       elsif char_count_r=x"7" then lcd_cmd_data_c <= x"3" & second(3 downto 0); 
       else lcd_cmd_data_c <= x"ff";
       end if;       
       lcd_wrdata_c   <= '1';
       lcd_wrcmd_c    <= '0';
       char_count_c   <= char_count_r;
       wait_count_c   <= x"0000";
     
     when wait_write_data =>
      if (wait_count_r=x"c34f" and char_count_r=x"7") then -- 1 ms time-out
        state_c      <= idle; 
        char_count_c <= char_count_r;
      elsif (wait_count_r=x"c34f" and char_count_r<x"7") then 
        state_c      <= write_cmd;
        char_count_c <= char_count_r+1;
      else 
        state_c      <= wait_write_data;
        char_count_c <= char_count_r;
      end if;
      lcd_cmd_data_c <= lcd_cmd_data_r;
      lcd_wrdata_c   <= '0';
      lcd_wrcmd_c    <= '0';
      wait_count_c   <= wait_count_r+1;
   end case;
  
 end process SMCombinatorial;
------------------------------------------------------------------------------------

------------------------------------------------------------------------------------
-- Infer Registers
------------------------------------------------------------------------------------
  RegisterProcess : process (arst, clk)
  begin
    if arst='1' then
      state_r        <= idle;
      lcd_cmd_data_r <= x"00";
      lcd_wrdata_r   <= '0';
      lcd_wrcmd_r    <= '0';
      char_count_r   <= x"0";
      wait_count_r   <= x"0000";
    elsif clk'event and clk='1' then
      state_r        <= state_c;
      lcd_cmd_data_r <= lcd_cmd_data_c;
      lcd_wrdata_r   <= lcd_wrdata_c;
      lcd_wrcmd_r    <= lcd_wrcmd_c;
      char_count_r   <= char_count_c;
      wait_count_r   <= wait_count_c;
    end if;
  end process RegisterProcess;
------------------------------------------------------------------------------------

------------------------------------------------------------------------------------
-- Drive output
------------------------------------------------------------------------------------
  lcd_cmd_data <= lcd_cmd_data_r;
  lcd_wrdata   <= lcd_wrdata_r;
  lcd_wrcmd    <= lcd_wrcmd_r;
  lcd_if_reset <= arst;
------------------------------------------------------------------------------------
end rtl;
--================================================================================== 