------------------------------------------------------------------------------
-- user_logic.vhd - entity/architecture pair
------------------------------------------------------------------------------
--
-- ***************************************************************************
-- ** Copyright (c) 1995-2006 Xilinx, Inc.  All rights reserved.            **
-- **                                                                       **
-- ** Xilinx, Inc.                                                          **
-- ** XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"         **
-- ** AS A COURTESY TO YOU, SOLELY FOR USE IN DEVELOPING PROGRAMS AND       **
-- ** SOLUTIONS FOR XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE,        **
-- ** OR INFORMATION AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE,        **
-- ** APPLICATION OR STANDARD, XILINX IS MAKING NO REPRESENTATION           **
-- ** THAT THIS IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,     **
-- ** AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE      **
-- ** FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY              **
-- ** WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE               **
-- ** IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR        **
-- ** REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF       **
-- ** INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS       **
-- ** FOR A PARTICULAR PURPOSE.                                             **
-- **                                                                       **
-- ***************************************************************************
--
------------------------------------------------------------------------------
-- Filename:          user_logic.vhd
-- Version:           1.00.c
-- Description:       User logic.
-- VHDL Standard:     VHDL'93
------------------------------------------------------------------------------
-- Naming Conventions:
--   active low signals:                    "*_n"
--   clock signals:                         "clk", "clk_div#", "clk_#x"
--   reset signals:                         "rst", "rst_n"
--   generics:                              "C_*"
--   user defined types:                    "*_TYPE"
--   state machine next state:              "*_ns"
--   state machine current state:           "*_cs"
--   combinatorial signals:                 "*_com"
--   pipelined or register delay signals:   "*_d#"
--   counter signals:                       "*cnt*"
--   clock enable signals:                  "*_ce"
--   internal version of output port:       "*_i"
--   device pins:                           "*_pin"
--   ports:                                 "- Names begin with Uppercase"
--   processes:                             "*_PROCESS"
--   component instantiations:              "<ENTITY_>I_<#|FUNC>"
------------------------------------------------------------------------------

-- DO NOT EDIT BELOW THIS LINE --------------------
library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

library proc_common_v2_00_a;
use proc_common_v2_00_a.proc_common_pkg.all;
-- DO NOT EDIT ABOVE THIS LINE --------------------

--USER libraries added here

------------------------------------------------------------------------------
-- Entity section
------------------------------------------------------------------------------
-- Definition of Generics:
--   C_LCD_TC                     -- minimum cycle time
--   C_LCD_TW                     -- minimum enable pulse width (Hi or Lo)
--   C_LCD_TSU1                   -- minimum control setup time
--   C_LCD_TH1                    -- minimum control hold time
--   C_LCD_TSU2                   -- minimum data setup time
--   C_LCD_TH2                    -- minimum data hold
--   C_LCD_DWIDTH                 -- Number of LCD data bits 
--   C_DWIDTH                     -- User logic data bus width
--   C_NUM_CE                     -- User logic chip enable bus width
--
-- Definition of Ports:
--   Bus2IP_Clk                   -- Bus to IP clock
--   Bus2IP_Reset                 -- Bus to IP reset
--   Bus2IP_Data                  -- Bus to IP data bus for user logic
--   Bus2IP_BE                    -- Bus to IP byte enables for user logic
--   Bus2IP_RdCE                  -- Bus to IP read chip enable for user logic
--   Bus2IP_WrCE                  -- Bus to IP write chip enable for user logic
--   IP2Bus_Data                  -- IP to Bus data bus for user logic
--   IP2Bus_Ack                   -- IP to Bus acknowledgement
--   IP2Bus_Retry                 -- IP to Bus retry response
--   IP2Bus_Error                 -- IP to Bus error response
--   IP2Bus_ToutSup               -- IP to Bus timeout suppress
--   IP2Bus_PostedWrInh           -- IP to Bus posted write inhibit
------------------------------------------------------------------------------

entity user_logic is
  generic
  (
    -- ADD USER GENERICS BELOW THIS LINE ---------------
    C_LCD_DWIDTH      : integer  := 4;
    C_CLK_PERIOD      : integer  := 10000;
    C_LCD_TC          : integer  := 500000;
    C_LCD_TW          : integer  := 230000;
    C_LCD_TSU1        : integer  := 40000;
    C_LCD_TH1         : integer  := 10000;
    C_LCD_TSU2        : integer  := 80000;
    C_LCD_TH2         : integer  := 10000;
    C_LCD_PWRON       : integer  := 40000000;
    C_LCD_INITD1      : integer  := 4100000;
    C_LCD_INITD2      : integer  := 100000;
    C_LCD_INITD3      : integer  := 1500000;
    C_LCD_CMDDLY      : integer  := 49000;
    C_LCD_SETCMD      : integer  := 40;
    C_LCD_ONCMD       : integer  := 12;
    C_LCD_CLRCMD      : integer  := 1;
    C_LCD_MODECMD     : integer  := 6;
    -- ADD USER GENERICS ABOVE THIS LINE ---------------

    -- DO NOT EDIT BELOW THIS LINE ---------------------
    -- Bus protocol parameters, do not add to or delete
    C_DWIDTH          : integer  := 32;
    C_NUM_CE          : integer  := 2
    -- DO NOT EDIT ABOVE THIS LINE ---------------------
  );
  port
  (
    -- ADD USER PORTS BELOW THIS LINE ------------------
	 LCD_E             : out std_logic;    
	 LCD_RW 	          : out std_logic;    
	 LCD_RS            : out std_logic;    
	 LCD_DB            : out std_logic_vector(C_LCD_DWIDTH-1 downto 0);
    -- ADD USER PORTS ABOVE THIS LINE ------------------

    -- DO NOT EDIT BELOW THIS LINE ---------------------
    -- Bus protocol ports, do not add to or delete
    Bus2IP_Clk                     : in  std_logic;
    Bus2IP_Reset                   : in  std_logic;
    Bus2IP_Data                    : in  std_logic_vector(0 to C_DWIDTH-1);
    Bus2IP_BE                      : in  std_logic_vector(0 to C_DWIDTH/8-1);
    Bus2IP_RdCE                    : in  std_logic_vector(0 to C_NUM_CE-1);
    Bus2IP_WrCE                    : in  std_logic_vector(0 to C_NUM_CE-1);
    IP2Bus_Data                    : out std_logic_vector(0 to C_DWIDTH-1);
    IP2Bus_Ack                     : out std_logic;
    IP2Bus_Retry                   : out std_logic;
    IP2Bus_Error                   : out std_logic;
    IP2Bus_ToutSup                 : out std_logic;
    IP2Bus_PostedWrInh             : out std_logic
    -- DO NOT EDIT ABOVE THIS LINE ---------------------
  );
end entity user_logic;

------------------------------------------------------------------------------
-- Architecture section
------------------------------------------------------------------------------

architecture IMP of user_logic is

  --USER signal declarations added here, as needed for user logic
constant CYCLE_WIDTH    : integer := C_LCD_TC/C_CLK_PERIOD - 1;
constant ENABLE_ON      : integer := C_LCD_TSU1/C_CLK_PERIOD - 1;
constant ENABLE_OFF     : integer := (C_LCD_TSU1 + C_LCD_TW)/C_CLK_PERIOD - 1;
constant TIMER_WIDTH    : integer := max2(1,log2(CYCLE_WIDTH));


  signal sig_Ack                        : std_logic;
  signal inProcess                      : std_logic;
  signal cmd_done                       : std_logic;

  signal init_done_i                    : std_logic;

  signal init_val                       : std_logic_vector(0 to 7);
  signal write_val                      : std_logic_vector(0 to 7);

  signal init_wr_req                    : std_logic;
  signal slv_wr_req                     : std_logic;
  signal write_req                      : std_logic;

  signal slv_isData			             : std_logic;
  signal isData   			             : std_logic;

  signal slv_reg                        : std_logic_vector(0 to C_DWIDTH-1);
  signal slv_bufFree                    : std_logic;
  signal slv_bufUsed                    : std_logic;


  ------------------------------------------
  -- Signals for user logic slave model s/w accessible register example
  ------------------------------------------
-- There is only one data register.  Two decodes are used to select Command or Data mode
  signal slv_reg_write_select           : std_logic_vector(0 to 1);
  signal slv_reg_read_select            : std_logic_vector(0 to 1);
  signal slv_ip2bus_data                : std_logic_vector(0 to C_DWIDTH-1);
  signal slv_read_ack                   : std_logic;
  signal slv_write_ack                  : std_logic;

-----------------------------------------------------------------------------
-- Component declarations
-----------------------------------------------------------------------------
component init_sm
  generic
  (
    C_LCD_DWIDTH      : integer  := 4;
    C_CLK_PERIOD      : integer  := 10000;
    C_LCD_PWRON       : integer  := 40000000;
    C_LCD_INITD1      : integer  := 4100000;
    C_LCD_INITD2      : integer  := 100000;
    C_LCD_INITD3      : integer  := 1500000;
    C_LCD_CMDDLY      : integer  := 49000;
    C_LCD_SETCMD      : integer  := 40;
    C_LCD_ONCMD       : integer  := 12;
    C_LCD_CLRCMD      : integer  := 1;
    C_LCD_MODECMD     : integer  := 6
  );                                                          
  port
  (
    Clk               : in  std_logic;
    Rst               : in  std_logic;
    Cmd_Done          : in  std_logic;
    Cmd_Req           : out std_logic;
    Cmd_Val           : out std_logic_vector(0 to 7);
    Init_Done         : out std_logic
  );
end component init_sm;

component cmd_sm is
  generic
  (
    C_LCD_DWIDTH      : integer  := 4;
    C_CLK_PERIOD      : integer  := 10000;
    C_LCD_TC          : integer  := 500000;
    C_LCD_TW          : integer  := 230000;
    C_LCD_TSU1        : integer  := 40000;
    C_LCD_TH1         : integer  := 10000;
    C_LCD_TSU2        : integer  := 80000;
    C_LCD_TH2         : integer  := 10000;
    C_LCD_CMDDLY      : integer  := 49000
  );                                                          
  port
  (
    Clk               : in  std_logic;
    Rst               : in  std_logic;
    Wr_Req            : in  std_logic;
    Wr_Val            : in  std_logic_vector(0 to 7);
    Req_Done          : out std_logic;
    Is4Bit            : in  std_logic;
    IsData            : in  std_logic;
	 LCD_E             : out std_logic;    
	 LCD_RW 	          : out std_logic;    
	 LCD_RS            : out std_logic;    
	 LCD_DB            : out std_logic_vector(C_LCD_DWIDTH-1 downto 0)
  );
end component cmd_sm;


begin

  --USER logic implementation added here

  ------------------------------------------
  -- Example code to read/write user logic slave model s/w accessible registers
  -- 
  -- Note:
  -- The example code presented here is to show you one way of reading/writing
  -- software accessible registers implemented in the user logic slave model.
  -- Each bit of the Bus2IP_WrCE/Bus2IP_RdCE signals is configured to correspond
  -- to one software accessible register by the top level template. For example,
  -- if you have four 32 bit software accessible registers in the user logic, you
  -- are basically operating on the following memory mapped registers:
  -- 
  --    Bus2IP_WrCE or   Memory Mapped
  --       Bus2IP_RdCE   Register
  --            "10"     C_BASEADDR + 0x0
  --            "01"     C_BASEADDR + 0x4
  -- 
  ------------------------------------------
  slv_reg_write_select <= Bus2IP_WrCE(0 to 1);
  slv_reg_read_select  <= Bus2IP_RdCE(0 to 1);
  slv_write_ack        <= Bus2IP_WrCE(0) or Bus2IP_WrCE(1);
  slv_read_ack         <= Bus2IP_RdCE(0) or Bus2IP_RdCE(1);

  -- implement slave model register(s)
  slv_wr_req <= slv_reg_write_select(0) or slv_reg_write_select(1);

  SLAVE_REG_WRITE_PROC : process( Bus2IP_Clk ) is
  begin

    if Bus2IP_Clk'event and Bus2IP_Clk = '1' then
      if Bus2IP_Reset = '1'  or init_done_i = '0' then
        slv_reg     <= (others => '0');
        slv_isData  <= '0';
        slv_bufFree <= '1';

      elsif slv_bufFree = '1' and slv_wr_req = '1' then
        slv_bufFree <= '0';
        case slv_reg_write_select is
          when "10" =>
       		slv_isData <= '1';   -- '1' for Data (offset 0x00)
            for byte_index in 0 to (C_DWIDTH/8)-1 loop
              if ( Bus2IP_BE(byte_index) = '1' ) then
                slv_reg(byte_index*8 to byte_index*8+7) <= Bus2IP_Data(byte_index*8 to byte_index*8+7);
              end if;
            end loop;
          when "01" =>
       		slv_isData <= '0';   -- '0' for Cmd (offset 0x04)
            for byte_index in 0 to (C_DWIDTH/8)-1 loop
              if ( Bus2IP_BE(byte_index) = '1' ) then
                slv_reg(byte_index*8 to byte_index*8+7) <= Bus2IP_Data(byte_index*8 to byte_index*8+7);
              end if;
            end loop;
          when others => 
            slv_isData <= slv_isData;
            slv_reg    <= slv_reg;
        end case;
      elsif slv_bufUsed = '1' then
        slv_bufFree <= '1';
      end if;
    end if;

  end process SLAVE_REG_WRITE_PROC;

  -- implement slave model register read mux
  SLAVE_REG_READ_PROC : process( slv_reg_read_select, slv_reg ) is
  begin
    case slv_reg_read_select is
      when "10" =>
        slv_ip2bus_data  <= slv_reg;
      when "01" =>
        slv_ip2bus_data  <= slv_reg;
      when others =>
        slv_ip2bus_data  <= (others => '0');
    end case;
  end process SLAVE_REG_READ_PROC;

  ------------------------------------------
  -- Example code to drive IP to Bus signals
  ------------------------------------------
  sig_Ack            <= init_done_i and slv_bufFree;

  IP2Bus_Data        <= slv_ip2bus_data;

  IP2Bus_Ack         <= sig_Ack;
  IP2Bus_Error       <= '0';
  IP2Bus_Retry       <= not sig_Ack;
  IP2Bus_ToutSup     <= '0';
  IP2Bus_PostedWrInh <= '1';

  ----------------------------------------
  --- LCD Controller
  ----------------------------------------

INIT_SM_I: init_sm
  generic map (
                C_LCD_DWIDTH   => C_LCD_DWIDTH,
                C_CLK_PERIOD   => C_CLK_PERIOD,
                C_LCD_PWRON    => C_LCD_PWRON, 
                C_LCD_INITD1   => C_LCD_INITD1,
                C_LCD_INITD2   => C_LCD_INITD2,
                C_LCD_INITD3   => C_LCD_INITD3,
                C_LCD_CMDDLY   => C_LCD_CMDDLY,
                C_LCD_SETCMD   => C_LCD_SETCMD,
                C_LCD_ONCMD    => C_LCD_ONCMD,
                C_LCD_CLRCMD   => C_LCD_CLRCMD,
                C_LCD_MODECMD  => C_LCD_MODECMD
              )
  port map    (
                Clk       => Bus2IP_Clk,
                Rst       => Bus2IP_Reset,
                Cmd_Done  => cmd_done,
                Cmd_Req   => init_wr_req,
                Cmd_Val   => init_val,
                Init_Done => init_done_i
              );

CMD_SM_I: cmd_sm
  generic map (
                C_LCD_DWIDTH   => C_LCD_DWIDTH,
                C_CLK_PERIOD   => C_CLK_PERIOD,
                C_LCD_TC       => C_LCD_TC,
                C_LCD_TW       => C_LCD_TW,
                C_LCD_TSU1     => C_LCD_TSU1,
                C_LCD_TH1      => C_LCD_TH1,
                C_LCD_TSU2     => C_LCD_TSU2,
                C_LCD_TH2      => C_LCD_TH2, 
                C_LCD_CMDDLY   => C_LCD_CMDDLY
              )
  port map    (
                Clk       => Bus2IP_Clk,
                Rst       => Bus2IP_Reset,
                Wr_Req    => write_req,
                Wr_Val    => write_val,
                Req_Done  => cmd_done,
                Is4Bit    => '1',
                IsData    => isData,
	             LCD_E     => LCD_E,
	             LCD_RW 	  => LCD_RW,
	             LCD_RS    => LCD_RS,
	             LCD_DB    => LCD_DB
              );

  START : process (Bus2IP_Clk) is
  begin
  	 if Bus2IP_Clk'event and Bus2IP_Clk = '1' then
   	if Bus2IP_Reset = '1' then
        inProcess     <= '0';
        isData        <= '0';
        write_req     <= '0';
        write_val     <= (others => '0');
        slv_bufUsed   <= '0';
		elsif inProcess = '0' then
        if init_done_i = '0' then
          isData      <= '0';
          write_req   <= init_wr_req;
          write_val   <= init_val;
        else
          isData      <= slv_isData;
          if slv_bufFree = '0' then
            write_req   <= '1';
            write_val   <= slv_reg(0 to 7);
            slv_bufUsed <= '1';
          end if;
        end if;

        if write_req = '1' then
           inProcess  <= '1';
           write_req  <= '0';
        end if;
      elsif cmd_done = '1' then
        inProcess   <= '0';
      elsif slv_bufUsed = '1' then
        slv_bufUsed <= '0';

      end if;
    end if;
  end process START;

end IMP;
